# Copyright (c) 2013, 2014 Dave Love, Liverpool University
# The licence for this file is as for the package itself.

# Fixme: provide manpages or more doc than the README pointer
# Fixme: different versions for switches?

# Uses hints from http://glennklockwood.blogspot.co.uk/2013/05/building-ipm-0983-for-lightweight-mpi.html

# Note that the github incarnation (see http://ipm-hpc.org) doesn't
# seem to do as muchor have a useful output parser.

%bcond_without mpich

Name:           ipm
Version:        0.983
Release:        5%{?dist}
Summary:        Portable profiling infrastructure for parallel codes

Group:          Development/System
License:        BSD
URL:            http://sourceforge.net/projects/ipm-hpc/
Source0:        http://downloads.sourceforge.net/project/ipm-hpc/ipm_%version.tgz
Source1:        ipm-README
Source2:	ipm.conf
Source3:	ipmrun.1
Source4:	ipm_parse.1
# alternative to the site-specific replacement mpirun in the source
# bin directory
Source5:	ipmrun
# fix failure with current openmpi
Patch1:         ipm-configure.patch
# ploticus <- pl consistent with packaging
Patch2:         ipm-ploticus.patch
# fix compilation error
Patch3:		ipm-ipm.patch
# support papi5
Patch4:		ipm-papi.patch
# ipm_parse fixes
Patch5:		ipm-parse.patch
# work with MPI3 headers
Patch6:		ipm-mpi3.patch
# fix -Wformat-security for fc21
Patch7:		ipm-format.patch

BuildRequires:  gcc-c++ gcc-gfortran openmpi-devel
BuildRequires:  papi-devel pnmpi-openmpi-devel
%if %{with mpich}
BuildRequires:	mpich-devel pnmpi-mpich-devel
%endif
# for use with ipm_parse
Requires:       ploticus
# Needed for key files?
Requires:       libipm-common = %{version}-%{release}
BuildRoot:      %{_tmppath}/%{name}-%{version}-%{release}-root-%(%{__id_u} -n)
ExclusiveArch: 	%{ix86} x86_64

%global desc\
IPM is a portable profiling infrastructure for parallel codes.  It\
provides a low-overhead performance profile of the performance aspects\
and resource utilization in a parallel program.  Communication,\
computation, and IO are the primary focus.  While the design scope\
targets production computing in HPC centers, IPM has found use in\
application development, performance debugging and parallel computing\
education.  The level of detail is selectable at runtime and presented\
through a variety of text and web reports.

%description
%decs

This package currently contains only the parsing utility for the
output of libipm.

# Split out to avoid the dependency on ploticus on compute nodes
%package -n libipm-openmpi
Summary:        Library files for %{name} - openmpi
Group:          Development/Libraries
Requires:	cpuid libipm-common = %{version}-%{release}

%description -n libipm-openmpi
%desc

This package contains the runtime library and module files for use
with openmpi.  Note that it doesn't supports functions added by MPI3.

%if %{with mpich}
%package -n libipm-mpich
Summary:        Library files for %{name} - mpich
Group:          Development/Libraries
Requires:	cpuid libipm-common = %{version}-%{release}

%description -n libipm-mpich
%desc

This package contains the runtime library and module files for use
with mpich.  Note that it doesn't supports functions added by MPI3.
%endif

%package -n libipm-common
Summary:        Common files for %{name}
BuildArch:	noarch

%description -n libipm-common
Common files for %{name}


%prep
%setup -q -n ipm
find -name CVS | xargs rm -r
cp %SOURCE1 README
%patch1 -p1 -b .mpi
%patch2 -p1 -b .pl
%patch3 -p1 -b .comp
%patch4 -p1 -b .papi
%patch5 -p1 -b .parse
%patch6 -p1 -b .mpi3
%patch7 -p1 -b .format

%build
%global do_build \
if [ %_arch = x86_64 ]; then\
  for cpu in OPTERON NEHALEM CORE2DUO; do\
    ./configure --with-arch=amd64 --with-os=LINUX --with-cpu=$cpu --switch=INFINIBAND --with-compiler=GNU CPPFLAGS="$CPPFLAGS -Werror=format-security" --with-hpm=PAPI\
    make shared\
    lccpu=$(echo $cpu|tr '[A-Z]' '[a-z]')\
    mv lib/libipm_%version.so libipm_$lccpu.so\
  done\
  # ./configure --with-arch=amd64 --with-os=LINUX --with-compiler=GNU --with-dynwrap\
  #  make shared\
else\
  ./configure --with-arch=x86 --with-os=LINUX --with-compiler=GNU --with-dynwrap\
fi\
make ipm
mkdir mpich openmpi
%if %{with mpich}
%{_mpich_load}
%do_build
mv *.so mpich
make clean
%{_mpich_unload}
%endif
%{_openmpi_load}
%do_build
mv *.so openmpi
%{_openmpi_unload}


%install
rm -rf $RPM_BUILD_ROOT
mkdir -p $RPM_BUILD_ROOT{%_bindir,%_includedir,%_sysconfdir,%_mandir/man1}
%global doinst \
mkdir -p $RPM_BUILD_ROOT$MPI_LIB/pnmpi-modules $RPM_BUILD_ROOT$MPI_BIN\
install libipm_*.so $RPM_BUILD_ROOT$MPI_LIB\
for f in libipm_*.so; do\
  pnmpi-patch $f $RPM_BUILD_ROOT$MPI_LIB/pnmpi-modules/$f\
done

%{_openmpi_load}
pushd openmpi
%doinst
popd
%{_openmpi_unload}
%if %{with mpich}
%{_mpich_load}
pushd mpich
%doinst
popd
%{_mpich_unload}
%endif
install -m755 bin/ipm_parse $RPM_BUILD_ROOT%_bindir
install ipm_key $RPM_BUILD_ROOT%_sysconfdir
cp %SOURCE2 $RPM_BUILD_ROOT%_sysconfdir
cp %SOURCE3 %SOURCE4 $RPM_BUILD_ROOT%_mandir/man1
# alternative to the site-specific replacement mpirun in the source
# bin directory
install -m755 %SOURCE5 $RPM_BUILD_ROOT%_bindir
rm -rf doc/CVS


%clean
rm -rf $RPM_BUILD_ROOT


%files
%_bindir/*
%_mandir/man1/*
%_sysconfdir/*

%files -n libipm-openmpi
%doc README CHANGELOG AUTHORS LICENSE COPYING WARNING doc/*
%_libdir/openmpi/lib/*.so
%dir %_libdir/openmpi/lib/pnmpi-modules
%_libdir/openmpi/lib/pnmpi-modules/*

%if %{with mpich}
%files -n libipm-mpich
%doc README CHANGELOG AUTHORS LICENSE COPYING WARNING doc/*
%_libdir/mpich/lib/*.so
%dir %_libdir/mpich/lib/pnmpi-modules
%_libdir/mpich/lib/pnmpi-modules/*
%endif

%files -n libipm-common
%config(noreplace) %_sysconfdir/ipm.conf
%config(noreplace) %_sysconfdir/ipm_key


%changelog
* Fri Jan 23 2015 Dave Love <d.love@liverpool.ac.uk> - 0.983-5
- Fix -Wformat-security for fc21

* Mon Jan 12 2015 Dave Love <d.love@liverpool.ac.uk> - 0.983-4
- Update for RHEL 6.6 mpich
- Check for MPI_Status.count_lo in configure, assuming that's the
  right tag in mpich3
- Fix declarations for MPI-3
- Generate pnmpi version
- Make mpich optional
- Fix ipmrun man page

* Mon Oct 13 2014 Dave Love <d.love@liverpool.ac.uk> - 0.983-3
- ipm_parse fixes
- Install ipm_key in sysconfdir
- Mod ipmrun, ipm.conf
- Change URL

* Tue Feb 25 2014 Dave Love <d.love@liverpool.ac.uk> - 0.983-2
- Build multiple libraries
- Change runipm

* Wed Feb 19 2014 Dave Love <d.love@liverpool.ac.uk> - 2.0.3-1
- Update to current source with unused, differently-licensed source removed
- Install in openmpi tree and patch installation in etc subdir
- Add LD_PRELOAD shim and README with pointer to doc
- Split into multiple packages

* Wed Nov 27 2013 Dave Love <d.love@liverpool.ac.uk> - 2.0.2-1
- Initial packaging
